"""Attach signals to this app's models."""
# -*- coding: utf-8 -*-
import json
import logging

import channels.layers
from asgiref.sync import async_to_sync

from django.db.models.signals import post_save
from django.dispatch import receiver

from .models import Job, Log


logger = logging.getLogger(__name__)  # pylint: disable=C0103


def send_message(event):
    '''
    Call back function to send message to the browser
    '''
    message = event['text']
    channel_layer = channels.layers.get_channel_layer()
    # Send message to WebSocket
    async_to_sync(channel_layer.send)(text_data=json.dumps(
        message
    ))


@receiver(post_save, sender=Job, dispatch_uid='update_job_status_listeners')
def update_job_status_listeners(sender, instance, **kwargs):
    '''
    Sends job status to the browser when a Job is modified
    '''

    logger.debug("Job modified: {} :: status = {}.".format(
        instance, instance.status))

    user = instance.owner
    group_name = 'job-user-{}'.format(user.username)

    message = {
        'job_id': instance.id,
        'title': instance.title,
        'status': instance.status,
        'modified': instance.modified.isoformat(),
    }

    channel_layer = channels.layers.get_channel_layer()

    async_to_sync(channel_layer.group_send)(
        group_name,
        {
            'type': 'send_message',
            'text': message
        }
    )


@receiver(post_save, sender=Log, dispatch_uid='update_job_log_listeners')
def update_job_log_listeners(sender, instance, **kwargs):
    '''
    Sends job status to the browser when a Log is modified
    '''

    logger.debug("Log modified: {} :: content = {}.".format(
        instance, instance.content))

    job_pk = instance.job.id
    group_name = 'job-log-{}'.format(job_pk)

    message = {
        'log_id': instance.id,
        'time': instance.time.isoformat(),
        'content': instance.content,
        'stream': instance.stream,
    }

    channel_layer = channels.layers.get_channel_layer()

    async_to_sync(channel_layer.group_send)(
        group_name,
        {
            'type': 'send_message',
            'text': message
        }
    )



# from channels.routing import route
# from .consumers import (
#     ws_job_status_connect, ws_job_status_disconnect,
#     ws_job_log_connect, ws_job_log_disconnect,
# )


# channel_routing = [
#     route('websocket.connect', ws_job_status_connect,       path=r'^/job-user/$'),
#     route('websocket.disconnect', ws_job_status_disconnect, path=r'^/job-user/$'),
#     route('websocket.connect', ws_job_log_connect,       path=r'^/job-log/(?P<job_pk>[0-9]+)/$'),
#     route('websocket.disconnect', ws_job_log_disconnect, path=r'^/job-log/(?P<job_pk>[0-9]+)/$'),
# ]


from django.urls import path
from channels.routing import ProtocolTypeRouter, URLRouter
from channels.auth import AuthMiddlewareStack

from .consumers import JobUserConsumer, JobLogConsumer


application = ProtocolTypeRouter({
    "websocket": AuthMiddlewareStack(
        URLRouter([
            path('ws/job-user/', JobUserConsumer),
            path('ws/job-log/<int:job_pk>/', JobLogConsumer),
        ]),
    ),

})


# """Manage websocket connections."""

import json
from channels.generic.websocket import AsyncWebsocketConsumer

from .models import Job


class JobUserConsumer(AsyncWebsocketConsumer):

    async def connect(self):
        '''
        Creates group, add to the valid channels
        Connects and sends to the browser the last jobs
        '''
        user = self.scope["user"]
        self.group_name = 'job-user-{}'.format(user.username)

        await self.channel_layer.group_add(
            self.group_name,
            self.channel_name
        )

        await self.accept()
        await self.send_last_jobs(user)

    async def disconnect(self, close_code):

        await self.channel_layer.group_discard(
            self.group_name,
            self.channel_name
        )

    async def send_last_jobs(self, user):

        last_jobs = user.jobs.order_by('-modified')[:10]

        for job in last_jobs:

            message = {
                'job_id': job.id,
                'title': job.title,
                'status': job.status,
                'modified': job.modified.isoformat(),
            }

            await self.channel_layer.group_send(
                self.group_name,
                {
                    'type': 'send_message',
                    'text': message
                }
            )

    async def send_message(self, event):
        message = event['text']

        # Send message to WebSocket
        await self.send(text_data=json.dumps(
            message
        ))


class JobLogConsumer(AsyncWebsocketConsumer):

    async def connect(self):
        '''
        Creates group, add to the valid channels
        Connects and sends to the browser the log
        '''
        job_pk = self.scope['url_route']['kwargs']['job_pk']
        self.group_name = 'job-log-{}'.format(job_pk)

        await self.channel_layer.group_add(
            self.group_name,
            self.channel_name
        )

        await self.accept()
        await self.send_log(job_pk)

    async def disconnect(self, close_code):

        await self.channel_layer.group_discard(
            self.group_name,
            self.channel_name
        )

    async def send_log(self, job_pk):

        job = Job.objects.get(pk=job_pk)
        logs = job.logs.order_by('time')

        for log in logs:
            message = {
                'log_id': log.id,
                'time': log.time.isoformat(),
                'content': log.content,
                'stream': log.stream,
            }

            await self.channel_layer.group_send(
                self.group_name,
                {
                    'type': 'send_message',
                    'text': message
                }
            )

    async def send_message(self, event):
        message = event['text']

        # Send message to WebSocket
        await self.send(text_data=json.dumps(
            message
        ))
